from django.contrib.gis.db.models.fields import MultiPolygonField
from django.db import models


class SimplifiedLandPolygon(models.Model):
    """
    This is a "simplified polygons" layer from OSM
    """

    geom = MultiPolygonField(srid=3857)



from django.contrib.gis.db.models import GeometryField
from django.db import models

"""
These models are autogenerated from the `osm2pgsql` command
"""


class PlanetOsmLine(models.Model):
    unique_id = models.BigIntegerField(primary_key=True)
    osm_id = models.BigIntegerField(blank=True, null=True)
    access = models.CharField(max_length=1024, blank=True, null=True)
    addr_housename = models.CharField(max_length=1024, db_column="addr:housename", blank=True, null=True)  # Field renamed to remove unsuitable characters.
    addr_housenumber = models.CharField(max_length=1024, db_column="addr:housenumber", blank=True, null=True)  # Field renamed to remove unsuitable characters.
    addr_interpolation = models.CharField(max_length=1024, db_column="addr:interpolation", blank=True, null=True)  # Field renamed to remove unsuitable characters.
    admin_level = models.CharField(max_length=1024, blank=True, null=True)
    aerialway = models.CharField(max_length=1024, blank=True, null=True)
    aeroway = models.CharField(max_length=1024, blank=True, null=True)
    amenity = models.CharField(max_length=1024, blank=True, null=True)
    barrier = models.CharField(max_length=1024, blank=True, null=True)
    bicycle = models.CharField(max_length=1024, blank=True, null=True)
    bridge = models.CharField(max_length=1024, blank=True, null=True)
    boundary = models.CharField(max_length=1024, blank=True, null=True)
    building = models.CharField(max_length=1024, blank=True, null=True)
    construction = models.CharField(max_length=1024, blank=True, null=True)
    covered = models.CharField(max_length=1024, blank=True, null=True)
    foot = models.CharField(max_length=1024, blank=True, null=True)
    highway = models.CharField(max_length=1024, blank=True, null=True)
    historic = models.CharField(max_length=1024, blank=True, null=True)
    horse = models.CharField(max_length=1024, blank=True, null=True)
    junction = models.CharField(max_length=1024, blank=True, null=True)
    landuse = models.CharField(max_length=1024, blank=True, null=True)
    layer = models.IntegerField(blank=True, null=True)
    leisure = models.CharField(max_length=1024, blank=True, null=True)
    lock = models.CharField(max_length=1024, blank=True, null=True)
    man_made = models.CharField(max_length=1024, blank=True, null=True)
    military = models.CharField(max_length=1024, blank=True, null=True)
    name = models.CharField(max_length=1024, blank=True, null=True)
    natural = models.CharField(max_length=1024, blank=True, null=True)
    oneway = models.CharField(max_length=1024, blank=True, null=True)
    place = models.CharField(max_length=1024, blank=True, null=True)
    power = models.CharField(max_length=1024, blank=True, null=True)
    railway = models.CharField(max_length=1024, blank=True, null=True)
    ref = models.CharField(max_length=1024, blank=True, null=True)
    religion = models.CharField(max_length=1024, blank=True, null=True)
    route = models.CharField(max_length=1024, blank=True, null=True)
    service = models.CharField(max_length=1024, blank=True, null=True)
    shop = models.CharField(max_length=1024, blank=True, null=True)
    surface = models.CharField(max_length=1024, blank=True, null=True)
    tourism = models.CharField(max_length=1024, blank=True, null=True)
    tracktype = models.CharField(max_length=1024, blank=True, null=True)
    tunnel = models.CharField(max_length=1024, blank=True, null=True)
    water = models.CharField(max_length=1024, blank=True, null=True)
    waterway = models.CharField(max_length=1024, blank=True, null=True)
    way_area = models.FloatField(blank=True, null=True)
    z_order = models.IntegerField(blank=True, null=True)
    way = GeometryField(blank=True, null=True)

    class Meta:
        managed = False
        db_table = "planet_osm_line"


class PlanetOsmPoint(models.Model):
    unique_id = models.BigIntegerField(primary_key=True)
    osm_id = models.BigIntegerField(blank=True, null=True)
    access = models.CharField(max_length=1024, blank=True, null=True)
    addr_housename = models.CharField(max_length=1024, db_column="addr:housename", blank=True, null=True)  # Field renamed to remove unsuitable characters.
    addr_housenumber = models.CharField(max_length=1024, db_column="addr:housenumber", blank=True, null=True)  # Field renamed to remove unsuitable characters.
    admin_level = models.CharField(max_length=1024, blank=True, null=True)
    aerialway = models.CharField(max_length=1024, blank=True, null=True)
    aeroway = models.CharField(max_length=1024, blank=True, null=True)
    amenity = models.CharField(max_length=1024, blank=True, null=True)
    barrier = models.CharField(max_length=1024, blank=True, null=True)
    boundary = models.CharField(max_length=1024, blank=True, null=True)
    building = models.CharField(max_length=1024, blank=True, null=True)
    highway = models.CharField(max_length=1024, blank=True, null=True)
    historic = models.CharField(max_length=1024, blank=True, null=True)
    junction = models.CharField(max_length=1024, blank=True, null=True)
    landuse = models.CharField(max_length=1024, blank=True, null=True)
    layer = models.IntegerField(blank=True, null=True)
    leisure = models.CharField(max_length=1024, blank=True, null=True)
    lock = models.CharField(max_length=1024, blank=True, null=True)
    man_made = models.CharField(max_length=1024, blank=True, null=True)
    military = models.CharField(max_length=1024, blank=True, null=True)
    name = models.CharField(max_length=1024, blank=True, null=True)
    natural = models.CharField(max_length=1024, blank=True, null=True)
    oneway = models.CharField(max_length=1024, blank=True, null=True)
    place = models.CharField(max_length=1024, blank=True, null=True)
    power = models.CharField(max_length=1024, blank=True, null=True)
    railway = models.CharField(max_length=1024, blank=True, null=True)
    ref = models.CharField(max_length=1024, blank=True, null=True)
    religion = models.CharField(max_length=1024, blank=True, null=True)
    shop = models.CharField(max_length=1024, blank=True, null=True)
    tourism = models.CharField(max_length=1024, blank=True, null=True)
    water = models.CharField(max_length=1024, blank=True, null=True)
    waterway = models.CharField(max_length=1024, blank=True, null=True)
    way = GeometryField(blank=True, null=True)

    class Meta:
        managed = False
        db_table = "planet_osm_point"


class PlanetOsmPolygon(models.Model):
    unique_id = models.BigIntegerField(primary_key=True)
    osm_id = models.BigIntegerField(blank=True, null=True)
    access = models.CharField(max_length=1024, blank=True, null=True)
    addr_housename = models.CharField(max_length=1024, db_column="addr:housename", blank=True, null=True)  # Field renamed to remove unsuitable characters.
    addr_housenumber = models.CharField(max_length=1024, db_column="addr:housenumber", blank=True, null=True)  # Field renamed to remove unsuitable characters.
    addr_interpolation = models.CharField(max_length=1024, db_column="addr:interpolation", blank=True, null=True)  # Field renamed to remove unsuitable characters.
    admin_level = models.CharField(max_length=1024, blank=True, null=True)
    aerialway = models.CharField(max_length=1024, blank=True, null=True)
    aeroway = models.CharField(max_length=1024, blank=True, null=True)
    amenity = models.CharField(max_length=1024, blank=True, null=True)
    barrier = models.CharField(max_length=1024, blank=True, null=True)
    bicycle = models.CharField(max_length=1024, blank=True, null=True)
    bridge = models.CharField(max_length=1024, blank=True, null=True)
    boundary = models.CharField(max_length=1024, blank=True, null=True)
    building = models.CharField(max_length=1024, blank=True, null=True)
    construction = models.CharField(max_length=1024, blank=True, null=True)
    covered = models.CharField(max_length=1024, blank=True, null=True)
    foot = models.CharField(max_length=1024, blank=True, null=True)
    highway = models.CharField(max_length=1024, blank=True, null=True)
    historic = models.CharField(max_length=1024, blank=True, null=True)
    horse = models.CharField(max_length=1024, blank=True, null=True)
    junction = models.CharField(max_length=1024, blank=True, null=True)
    landuse = models.CharField(max_length=1024, blank=True, null=True)
    layer = models.IntegerField(blank=True, null=True)
    leisure = models.CharField(max_length=1024, blank=True, null=True)
    lock = models.CharField(max_length=1024, blank=True, null=True)
    man_made = models.CharField(max_length=1024, blank=True, null=True)
    military = models.CharField(max_length=1024, blank=True, null=True)
    name = models.CharField(max_length=1024, blank=True, null=True)
    natural = models.CharField(max_length=1024, blank=True, null=True)
    oneway = models.CharField(max_length=1024, blank=True, null=True)
    place = models.CharField(max_length=1024, blank=True, null=True)
    power = models.CharField(max_length=1024, blank=True, null=True)
    railway = models.CharField(max_length=1024, blank=True, null=True)
    ref = models.CharField(max_length=1024, blank=True, null=True)
    religion = models.CharField(max_length=1024, blank=True, null=True)
    route = models.CharField(max_length=1024, blank=True, null=True)
    service = models.CharField(max_length=1024, blank=True, null=True)
    shop = models.CharField(max_length=1024, blank=True, null=True)
    surface = models.CharField(max_length=1024, blank=True, null=True)
    tourism = models.CharField(max_length=1024, blank=True, null=True)
    tracktype = models.CharField(max_length=1024, blank=True, null=True)
    tunnel = models.CharField(max_length=1024, blank=True, null=True)
    water = models.CharField(max_length=1024, blank=True, null=True)
    waterway = models.CharField(max_length=1024, blank=True, null=True)
    way_area = models.FloatField(blank=True, null=True)
    z_order = models.IntegerField(blank=True, null=True)
    way = GeometryField(blank=True, null=True)

    class Meta:
        managed = False
        db_table = "planet_osm_polygon"



from djangostreetmap import models
from django.contrib.gis import admin


@admin.register(models.SimplifiedLandPolygon)
class SimplifiedLandPolygonAdmin(admin.GISModelAdmin):  # type: ignore
    pass



# Generated by Django 3.1.13 on 2021-09-03 07:14

from typing import List, Tuple

import django.contrib.gis.db.models.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []  # type: List[Tuple[str, str]]

    operations = [
        migrations.CreateModel(
            name="PlanetOsmLine",
            fields=[
                ("unique_id", models.BigIntegerField(primary_key=True, serialize=False)),
                ("osm_id", models.BigIntegerField(blank=True, null=True)),
                ("access", models.CharField(blank=True, max_length=1024, null=True)),
                ("addr_housename", models.CharField(blank=True, db_column="addr:housename", max_length=1024, null=True)),
                ("addr_housenumber", models.CharField(blank=True, db_column="addr:housenumber", max_length=1024, null=True)),
                ("addr_interpolation", models.CharField(blank=True, db_column="addr:interpolation", max_length=1024, null=True)),
                ("admin_level", models.CharField(blank=True, max_length=1024, null=True)),
                ("aerialway", models.CharField(blank=True, max_length=1024, null=True)),
                ("aeroway", models.CharField(blank=True, max_length=1024, null=True)),
                ("amenity", models.CharField(blank=True, max_length=1024, null=True)),
                ("barrier", models.CharField(blank=True, max_length=1024, null=True)),
                ("bicycle", models.CharField(blank=True, max_length=1024, null=True)),
                ("bridge", models.CharField(blank=True, max_length=1024, null=True)),
                ("boundary", models.CharField(blank=True, max_length=1024, null=True)),
                ("building", models.CharField(blank=True, max_length=1024, null=True)),
                ("construction", models.CharField(blank=True, max_length=1024, null=True)),
                ("covered", models.CharField(blank=True, max_length=1024, null=True)),
                ("foot", models.CharField(blank=True, max_length=1024, null=True)),
                ("highway", models.CharField(blank=True, max_length=1024, null=True)),
                ("historic", models.CharField(blank=True, max_length=1024, null=True)),
                ("horse", models.CharField(blank=True, max_length=1024, null=True)),
                ("junction", models.CharField(blank=True, max_length=1024, null=True)),
                ("landuse", models.CharField(blank=True, max_length=1024, null=True)),
                ("layer", models.IntegerField(blank=True, null=True)),
                ("leisure", models.CharField(blank=True, max_length=1024, null=True)),
                ("lock", models.CharField(blank=True, max_length=1024, null=True)),
                ("man_made", models.CharField(blank=True, max_length=1024, null=True)),
                ("military", models.CharField(blank=True, max_length=1024, null=True)),
                ("name", models.CharField(blank=True, max_length=1024, null=True)),
                ("natural", models.CharField(blank=True, max_length=1024, null=True)),
                ("oneway", models.CharField(blank=True, max_length=1024, null=True)),
                ("place", models.CharField(blank=True, max_length=1024, null=True)),
                ("power", models.CharField(blank=True, max_length=1024, null=True)),
                ("railway", models.CharField(blank=True, max_length=1024, null=True)),
                ("ref", models.CharField(blank=True, max_length=1024, null=True)),
                ("religion", models.CharField(blank=True, max_length=1024, null=True)),
                ("route", models.CharField(blank=True, max_length=1024, null=True)),
                ("service", models.CharField(blank=True, max_length=1024, null=True)),
                ("shop", models.CharField(blank=True, max_length=1024, null=True)),
                ("surface", models.CharField(blank=True, max_length=1024, null=True)),
                ("tourism", models.CharField(blank=True, max_length=1024, null=True)),
                ("tracktype", models.CharField(blank=True, max_length=1024, null=True)),
                ("tunnel", models.CharField(blank=True, max_length=1024, null=True)),
                ("water", models.CharField(blank=True, max_length=1024, null=True)),
                ("waterway", models.CharField(blank=True, max_length=1024, null=True)),
                ("way_area", models.FloatField(blank=True, null=True)),
                ("z_order", models.IntegerField(blank=True, null=True)),
                ("way", django.contrib.gis.db.models.fields.GeometryField(blank=True, null=True, srid=4326)),
            ],
            options={
                "db_table": "planet_osm_line",
                "managed": False,
            },
        ),
        migrations.CreateModel(
            name="PlanetOsmPoint",
            fields=[
                ("unique_id", models.BigIntegerField(primary_key=True, serialize=False)),
                ("osm_id", models.BigIntegerField(blank=True, null=True)),
                ("access", models.CharField(blank=True, max_length=1024, null=True)),
                ("addr_housename", models.CharField(blank=True, db_column="addr:housename", max_length=1024, null=True)),
                ("addr_housenumber", models.CharField(blank=True, db_column="addr:housenumber", max_length=1024, null=True)),
                ("admin_level", models.CharField(blank=True, max_length=1024, null=True)),
                ("aerialway", models.CharField(blank=True, max_length=1024, null=True)),
                ("aeroway", models.CharField(blank=True, max_length=1024, null=True)),
                ("amenity", models.CharField(blank=True, max_length=1024, null=True)),
                ("barrier", models.CharField(blank=True, max_length=1024, null=True)),
                ("boundary", models.CharField(blank=True, max_length=1024, null=True)),
                ("building", models.CharField(blank=True, max_length=1024, null=True)),
                ("highway", models.CharField(blank=True, max_length=1024, null=True)),
                ("historic", models.CharField(blank=True, max_length=1024, null=True)),
                ("junction", models.CharField(blank=True, max_length=1024, null=True)),
                ("landuse", models.CharField(blank=True, max_length=1024, null=True)),
                ("layer", models.IntegerField(blank=True, null=True)),
                ("leisure", models.CharField(blank=True, max_length=1024, null=True)),
                ("lock", models.CharField(blank=True, max_length=1024, null=True)),
                ("man_made", models.CharField(blank=True, max_length=1024, null=True)),
                ("military", models.CharField(blank=True, max_length=1024, null=True)),
                ("name", models.CharField(blank=True, max_length=1024, null=True)),
                ("natural", models.CharField(blank=True, max_length=1024, null=True)),
                ("oneway", models.CharField(blank=True, max_length=1024, null=True)),
                ("place", models.CharField(blank=True, max_length=1024, null=True)),
                ("power", models.CharField(blank=True, max_length=1024, null=True)),
                ("railway", models.CharField(blank=True, max_length=1024, null=True)),
                ("ref", models.CharField(blank=True, max_length=1024, null=True)),
                ("religion", models.CharField(blank=True, max_length=1024, null=True)),
                ("shop", models.CharField(blank=True, max_length=1024, null=True)),
                ("tourism", models.CharField(blank=True, max_length=1024, null=True)),
                ("water", models.CharField(blank=True, max_length=1024, null=True)),
                ("waterway", models.CharField(blank=True, max_length=1024, null=True)),
                ("way", django.contrib.gis.db.models.fields.GeometryField(blank=True, null=True, srid=4326)),
            ],
            options={
                "db_table": "planet_osm_point",
                "managed": False,
            },
        ),
        migrations.CreateModel(
            name="PlanetOsmPolygon",
            fields=[
                ("unique_id", models.BigIntegerField(primary_key=True, serialize=False)),
                ("osm_id", models.BigIntegerField(blank=True, null=True)),
                ("access", models.CharField(blank=True, max_length=1024, null=True)),
                ("addr_housename", models.CharField(blank=True, db_column="addr:housename", max_length=1024, null=True)),
                ("addr_housenumber", models.CharField(blank=True, db_column="addr:housenumber", max_length=1024, null=True)),
                ("addr_interpolation", models.CharField(blank=True, db_column="addr:interpolation", max_length=1024, null=True)),
                ("admin_level", models.CharField(blank=True, max_length=1024, null=True)),
                ("aerialway", models.CharField(blank=True, max_length=1024, null=True)),
                ("aeroway", models.CharField(blank=True, max_length=1024, null=True)),
                ("amenity", models.CharField(blank=True, max_length=1024, null=True)),
                ("barrier", models.CharField(blank=True, max_length=1024, null=True)),
                ("bicycle", models.CharField(blank=True, max_length=1024, null=True)),
                ("bridge", models.CharField(blank=True, max_length=1024, null=True)),
                ("boundary", models.CharField(blank=True, max_length=1024, null=True)),
                ("building", models.CharField(blank=True, max_length=1024, null=True)),
                ("construction", models.CharField(blank=True, max_length=1024, null=True)),
                ("covered", models.CharField(blank=True, max_length=1024, null=True)),
                ("foot", models.CharField(blank=True, max_length=1024, null=True)),
                ("highway", models.CharField(blank=True, max_length=1024, null=True)),
                ("historic", models.CharField(blank=True, max_length=1024, null=True)),
                ("horse", models.CharField(blank=True, max_length=1024, null=True)),
                ("junction", models.CharField(blank=True, max_length=1024, null=True)),
                ("landuse", models.CharField(blank=True, max_length=1024, null=True)),
                ("layer", models.IntegerField(blank=True, null=True)),
                ("leisure", models.CharField(blank=True, max_length=1024, null=True)),
                ("lock", models.CharField(blank=True, max_length=1024, null=True)),
                ("man_made", models.CharField(blank=True, max_length=1024, null=True)),
                ("military", models.CharField(blank=True, max_length=1024, null=True)),
                ("name", models.CharField(blank=True, max_length=1024, null=True)),
                ("natural", models.CharField(blank=True, max_length=1024, null=True)),
                ("oneway", models.CharField(blank=True, max_length=1024, null=True)),
                ("place", models.CharField(blank=True, max_length=1024, null=True)),
                ("power", models.CharField(blank=True, max_length=1024, null=True)),
                ("railway", models.CharField(blank=True, max_length=1024, null=True)),
                ("ref", models.CharField(blank=True, max_length=1024, null=True)),
                ("religion", models.CharField(blank=True, max_length=1024, null=True)),
                ("route", models.CharField(blank=True, max_length=1024, null=True)),
                ("service", models.CharField(blank=True, max_length=1024, null=True)),
                ("shop", models.CharField(blank=True, max_length=1024, null=True)),
                ("surface", models.CharField(blank=True, max_length=1024, null=True)),
                ("tourism", models.CharField(blank=True, max_length=1024, null=True)),
                ("tracktype", models.CharField(blank=True, max_length=1024, null=True)),
                ("tunnel", models.CharField(blank=True, max_length=1024, null=True)),
                ("water", models.CharField(blank=True, max_length=1024, null=True)),
                ("waterway", models.CharField(blank=True, max_length=1024, null=True)),
                ("way_area", models.FloatField(blank=True, null=True)),
                ("z_order", models.IntegerField(blank=True, null=True)),
                ("way", django.contrib.gis.db.models.fields.GeometryField(blank=True, null=True, srid=4326)),
            ],
            options={
                "db_table": "planet_osm_polygon",
                "managed": False,
            },
        ),
        migrations.CreateModel(
            name="PlanetOsmRoads",
            fields=[],
            options={
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("djangostreetmap.planetosmline",),
        ),
    ]



import os
import subprocess
import urllib.request
from dataclasses import dataclass

# osmupdate
from datetime import datetime
from pathlib import Path

default_path = Path("/tmp") / "osm"


@dataclass
class OSMUpdate:
    """
    Handle initial fetch and updates of a particular region
    from geofabrik

    """

    region: str
    country: str
    cache_dir: Path = Path("/tmp") / "osm"
    server: str = "download.geofabrik.de"

    @property
    def full_url(self):
        return f"http://{self.server}/{self.region}/{self.country}-latest.osm.pbf"

    @property
    def updates_url(self):
        return f"http://{self.server}/{self.region}/{self.country}-updates/"

    @property
    def get_cache_dir(self) -> Path:
        return Path() / (self.cache_dir or default_path) / self.region

    @property
    def get_cache_file(self) -> Path:
        return self.get_cache_dir / self.filename

    @property
    def get_temp_dir(self) -> Path:
        return self.get_cache_dir / f"{self.country}-updates"

    @property
    def filename(self) -> Path:
        return Path(f"{self.country}-latest.osm.pbf")

    @property
    def file_exists(self) -> bool:
        return os.path.exists(self.get_cache_file)

    def ensure_cache_dir(self):
        os.makedirs(self.get_cache_dir(), exist_ok=True)

    def fetch(self):
        # Ensure that the directory to save the file to exists
        os.makedirs(self.get_cache_dir, exist_ok=True)
        os.makedirs(self.get_temp_dir, exist_ok=True)
        if self.file_exists:
            # The file itself should not exist
            raise FileExistsError
        urllib.request.urlretrieve(self.full_url, self.get_cache_file)

    def update_file(self):
        """
        Calls osmupdate to pull the changes relevant for the area
        """
        try:
            now = datetime.today().strftime("%Y-%m-%d")
            cachefile = self.get_cache_file
            cmd = "osmupdate"
            params = [
                f"{cachefile}",
                str(cachefile).replace("latest", f"update-{now}"),
                f"--base-url={self.updates_url}",
                f"--tempfiles={self.get_temp_dir}/",
                "--keep-tempfiles",
                "--trust-tempfiles",
            ]
            print(" ".join([cmd, *params]))
            subprocess.call([cmd, *params])
        except Exception as E:  # noqa: F841
            raise


@dataclass
class EastTimor(OSMUpdate):
    region: str = "asia"
    country: str = "east-timor"



from django.urls import path
from django.views.decorators.cache import cache_page

from .views import Aeroways, ExampleMapView, Hospitals, LandLayer, MajorRoads, MinorRoads, BuildingPolygon

app_name = "djangostreetmap"

urlpatterns = [
    # path("highways/<int:zoom>/<int:x>/<int:y>.pbf", cache_page(60 * 15)(RoadLayerView.as_view()), name="highways"),
    # path("admboundary/<int:zoom>/<int:x>/<int:y>.pbf", cache_page(60 * 15)(AdminBoundaryLayerView.as_view()), name="admin"),
    # path("islands/<int:zoom>/<int:x>/<int:y>.pbf", cache_page(60 * 15)(IslandsAreaLayerView.as_view()), name="islands"),
    # path("facebookai/<int:zoom>/<int:x>/<int:y>.pbf", cache_page(60 * 15)(FacebookAiLayerView.as_view()), name="facebookai"),
    path("example_map", ExampleMapView.as_view()),
    # These allow maps to make reverse URL calls a little easier
    # path("highways/", RoadLayerView.as_view(), name="highways"),
    # path("admboundary/", AdminBoundaryLayerView.as_view(), name="admin"),
    # path("islands/", IslandsAreaLayerView.as_view(), name="islands"),
    # path("facebookai/", FacebookAiLayerView.as_view(), name="facebookai"),
    path("buildings/", BuildingPolygon.as_view(), name="buildings"),
    path("buildings/<int:zoom>/<int:x>/<int:y>.pbf", cache_page(60 * 15)(BuildingPolygon.as_view()), name="buildings"),
    path("roads/", MajorRoads.as_view(), name="roads"),
    path("roads/<int:zoom>/<int:x>/<int:y>.pbf", cache_page(60 * 15)(MajorRoads.as_view()), name="roads"),
    path("hospitals", Hospitals.as_view(), name="hospitals"),
    path("airports", Aeroways.as_view(), name="aeroways"),
    path("minor_roads/", MinorRoads.as_view(), name="minor_roads"),
    path("minor_roads/<int:zoom>/<int:x>/<int:y>.pbf", cache_page(60 * 15)(MinorRoads.as_view()), name="minor_roads"),
    path("land/", cache_page(60 * 15)(LandLayer.as_view()), name="land"),
    path("land/<int:zoom>/<int:x>/<int:y>.pbf", cache_page(60 * 15)(LandLayer.as_view()), name="land"),
]



from typing import Iterable, List, Optional
from django.apps import apps

from django.db import connection
from django.http import JsonResponse
from django.http.response import HttpResponse
from django.views import View
from django.views.generic.base import TemplateView
from django.contrib.gis.db.models.functions import Centroid, Transform
from dataclasses import asdict
from psycopg2 import sql
import logging
from osmflex.models import RoadLine
from djangostreetmap import models

from annotations import GeoJsonSerializer, MultiGeoJsonSerializer

from .tilegenerator import MvtQuery, Tile
from .timer import Timer

logger = logging.getLogger(__name__)


class ExampleMapView(TemplateView):
    template_name = "leaflet_tile_layers.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["map_view"] = "[-8.556416, 125.5524171], 14"
        return context


class TileLayerView(View):
    """
    Handle generation and views of tile layers
    For urls.py you likely want two entries for easier reversing in templates
    >>> path("mylayer/<int:zoom>/<int:x>/<int:y>.pbf", MyTileLayerView.as_view()), name="mylayer"),
    >>> path("mylayer", MyTileLayerView.as_view(), name="mylayer"),
    """

    layers: List[MvtQuery] = []

    def get_layers(self, tile: Tile) -> List[MvtQuery]:
        """
        Override for dynamically determining tile content
        based on tile properties (most likely zoom)
        """
        return self.layers

    def _generate_tile(self, tile: Tile) -> List[bytes]:

        params = asdict(tile)

        with connection.cursor() as cursor:
            tiles = []
            for layer in self.get_layers(tile):
                query = layer.as_mvt()
                # Uncomment to see the SQL which is run
                # logger.info(query.as_string(cursor.cursor))
                # with Timer(name="tile generator", logger=logger.info):
                tile_response = None  # type: Optional[Iterable]
                try:
                    cursor.execute(query, params)
                    tile_response = cursor.fetchone()
                except Exception as E:
                    logger.error(f"{E}")
                    logger.info(query.as_string(cursor.cursor))

                if tile_response:
                    content = tile_response[0]  # type: bytes
                    tiles += content

            return tiles

    def get(self, request, *args, **kwargs):
        with Timer(name="tile get", logger=logger.info):
            tile = Tile(**kwargs)  # Expect to receive zoom, x, and y in kwargs
            tiles = b"".join(self._generate_tile(tile))
            return HttpResponse(content=tiles, content_type="application/binary")


class BuildingPolygon(TileLayerView):
    def get_layers(self, tile: Tile):
        if tile.zoom >= 11:
            return [MvtQuery(table="osmflex_buildingpolygon", attributes=["name", "osm_id", "osm_subtype"], layer="buildings", pk="osm_id")]
        elif tile.zoom >= 6:
            return [MvtQuery(table="osmflex_buildingpolygon", attributes=["name", "osm_id", "osm_subtype"], centroid=True, layer="building_point", pk="osm_id")]
        else:
            return []


class MajorRoads(TileLayerView):
    layers = [MvtQuery(table="osmflex_roadmajor", attributes=["name", "osm_type"], layer="roads", pk="osm_id")]


class MinorRoads(TileLayerView):
    def get_layers(self, tile: Tile):
        layers = []
        for road_class, min_zoom in [
            ("trunk", 2),
            ("steps", 12),
            ("road", 12),
            ("footway", 12),
            ("secondary", 7),
            ("tertiary", 9),
            ("secondary_link", 7),
            ("tertiary_link", 9),
            ("living_street", 12),
            ("pedestrian", 12),
            ("primary", 5),
            ("residential", 13),
            ("primary_link", 5),
            ("track", 12),
            ("motorway_link", 12),
            ("motorway", 5),
            ("service", 12),
            ("unclassified", 12),
            ("path", 12),
        ]:
            if tile.zoom > min_zoom:
                layers.append(
                    MvtQuery(
                        table=RoadLine._meta.db_table,
                        attributes=["name", "osm_type"],
                        filters=sql.SQL("{} = {}").format(sql.Identifier("osm_type"), sql.Literal(road_class)),
                        layer=road_class,
                        transform=False,
                        pk="osm_id",
                    )
                )
            else:
                logger.debug("Out of zoom: %s", road_class)

        return layers


class Hospitals(View):
    def get(self, request, *args, **kwargs):
        AmenityPoint = apps.get_model("osmflex", "AmenityPoint")
        AmenityPolygon = apps.get_model("osmflex", "AmenityPolygon")

        points = AmenityPoint.objects.filter(osm_type__in=["hospital", "clinic"]).annotate(geom_for_json=Transform("geom", 4326))
        centroids = AmenityPolygon.objects.filter(osm_type__in=["hospital", "clinic"]).annotate(geom_centroid=Transform(Centroid("geom"), 4326))

        response = MultiGeoJsonSerializer(
            [
                GeoJsonSerializer(points, "geom_for_json", ("name", "osm_type", "address")),
                GeoJsonSerializer(centroids, "geom_centroid", ("name", "osm_type", "address")),
            ]
        )

        return JsonResponse(data=asdict(response.to_collection()), content_type="application/json")


class Aeroways(View):
    def get(self, request, *args, **kwargs):
        point = apps.get_model("osmflex", "InfrastructurePoint")
        polygon = apps.get_model("osmflex", "InfrastructurePolygon")

        points = point.objects.filter(osm_type__in=["aeroway"]).annotate(geom_for_json=Transform("geom", 4326))
        centroids = polygon.objects.filter(osm_type__in=["aeroway"]).annotate(geom_centroid=Transform(Centroid("geom"), 4326))

        response = MultiGeoJsonSerializer(
            [
                GeoJsonSerializer(points, "geom_for_json", ("name", "osm_type")),
                GeoJsonSerializer(centroids, "geom_centroid", ("name", "osm_type")),
            ]
        )

        return JsonResponse(data=asdict(response.to_collection()), content_type="application/json")


class LandLayer(TileLayerView):
    layers = [MvtQuery(table=models.SimplifiedLandPolygon._meta.db_table, layer="land")]



from django.core import management
from django.test import TestCase

# Create your tests here.


class MyTestCase(TestCase):
    def test_foo(self):
        management.call_command("import_highways", "djangostreetmap/papua-new-guinea-latest.osm.pbf", "highways")
        self.assertEqual(True, True)
